@@ -1,5 +1,9 @@
+import contextlib
 import os
 from typing import List
+from typing import Type
+from urllib.parse import urlparse
+from urllib.request import urlopen
 
 from dataconf import utils
 from dataconf.exceptions import MalformedConfigException
@@ -7,6 +11,10 @@ from pyhocon import ConfigFactory
 from pyhocon import HOCONConverter
 from pyhocon.config_parser import ConfigTree
 import pyparsing
+from yaml import safe_load
+
+HOCON = 1
+YAML = 2
 
 
 def parse(
@@ -20,8 +28,12 @@ def parse(
         )
 
 
-def env_dict_list(prefix: str):
-    return utils.__dict_list_parsing(prefix, os.environ)
+def env_vars_parse(*args, **kwargs):
+    return utils.__env_vars_parse(*args, **kwargs)
+
+
+def cli_parse(*args, **kwargs):
+    return utils.__cli_parse(*args, **kwargs)
 
 
 class Multi:
@@ -32,25 +44,46 @@ class Multi:
 
     def env(self, prefix: str, **kwargs) -> "Multi":
         self.strict = False
-        return self.dict(env_dict_list(prefix), **kwargs)
+        data = env_vars_parse(prefix, os.environ)
+        return self.dict(data, **kwargs)
 
     def dict(self, obj: str, **kwargs) -> "Multi":
         conf = ConfigFactory.from_dict(obj)
         return Multi(self.confs + [conf], self.strict, **kwargs)
 
-    def string(self, s: str, **kwargs) -> "Multi":
+    def string(self, s: str, loader: str = HOCON, **kwargs) -> "Multi":
+
+        if loader == YAML:
+            data = safe_load(s)
+            return self.dict(data, **kwargs)
+
         conf = ConfigFactory.parse_string(s)
         return Multi(self.confs + [conf], self.strict, **kwargs)
 
-    def url(self, uri: str, **kwargs) -> "Multi":
-        conf = ConfigFactory.parse_URL(uri)
+    def url(self, uri: str, timeout: int = 10, **kwargs) -> "Multi":
+        path = urlparse(uri).path
+        if path.endswith(".yaml") or path.endswith(".yml"):
+            with contextlib.closing(urlopen(uri, timeout=timeout)) as fd:
+                s = fd.read().decode("utf-8")
+            return self.string(s, loader=YAML, **kwargs)
+
+        conf = ConfigFactory.parse_URL(uri, timeout=timeout)
         return Multi(self.confs + [conf], self.strict, **kwargs)
 
     def file(self, path: str, **kwargs) -> "Multi":
+        if path.endswith(".yaml") or path.endswith(".yml"):
+            with open(path, "r") as f:
+                data = safe_load(f)
+            return self.dict(data, **kwargs)
+
         conf = ConfigFactory.parse_file(path)
         return Multi(self.confs + [conf], self.strict, **kwargs)
 
-    def on(self, clazz):
+    def cli(self, argv: List[str], **kwargs) -> "Multi":
+        data = cli_parse(argv)
+        return self.dict(data, **kwargs)
+
+    def on(self, clazz: Type):
         conf, *nxts = self.confs
         for nxt in nxts:
             conf = ConfigTree.merge_configs(conf, nxt)
@@ -60,31 +93,35 @@ class Multi:
 multi = Multi([])
 
 
-def env(prefix: str, clazz, **kwargs):
+def env(prefix: str, clazz: Type, **kwargs):
     return multi.env(prefix, **kwargs).on(clazz)
 
 
-def dict(obj: str, clazz, **kwargs):
+def dict(obj: str, clazz: Type, **kwargs):
     return multi.dict(obj, **kwargs).on(clazz)
 
 
-def string(s: str, clazz, **kwargs):
+def string(s: str, clazz: Type, **kwargs):
     return multi.string(s, **kwargs).on(clazz)
 
 
-def url(uri: str, clazz, **kwargs):
+def url(uri: str, clazz: Type, **kwargs):
     return multi.url(uri, **kwargs).on(clazz)
 
 
-def file(path: str, clazz, **kwargs):
+def file(path: str, clazz: Type, **kwargs):
     return multi.file(path, **kwargs).on(clazz)
 
 
-def load(path: str, clazz, **kwargs):
+def cli(argv: List[str], clazz: Type, **kwargs):
+    return multi.cli(argv, **kwargs).on(clazz)
+
+
+def load(path: str, clazz: Type, **kwargs):
     return file(path, clazz, **kwargs)
 
 
-def loads(s: str, clazz, **kwargs):
+def loads(s: str, clazz: Type, **kwargs):
     return string(s, clazz, **kwargs)
 
 
