@@ -3,10 +3,18 @@ from dataclasses import asdict
 from dataclasses import fields
 from dataclasses import is_dataclass
 from datetime import datetime
+from enum import Enum
+from enum import IntEnum
+from inspect import isclass
+from typing import Any
+from typing import Dict
 from typing import get_args
 from typing import get_origin
+from typing import List
+from typing import Type
 from typing import Union
 
+from dataconf.exceptions import AmbiguousSubclassException
 from dataconf.exceptions import EnvListOrderException
 from dataconf.exceptions import MalformedConfigException
 from dataconf.exceptions import MissingTypeException
@@ -24,7 +32,7 @@ import pyparsing
 NoneType = type(None)
 
 
-def __parse_type(value, clazz, path, check):
+def __parse_type(value: Any, clazz: Type, path: str, check: bool):
     try:
         if check:
             return value
@@ -34,13 +42,12 @@ def __parse_type(value, clazz, path, check):
     raise TypeConfigException(f"expected type {clazz} at {path}, got {type(value)}")
 
 
-def is_optional(type):
+def is_optional(type: Type):
     # Optional = Union[T, NoneType]
     return get_origin(type) is Union and NoneType in get_args(type)
 
 
-def __parse(value: any, clazz, path, strict, ignore_unexpected):
-
+def __parse(value: any, clazz: Type, path: str, strict: bool, ignore_unexpected: bool):
     if is_dataclass(clazz):
 
         if not isinstance(value, ConfigTree):
@@ -167,6 +174,22 @@ def __parse(value: any, clazz, path, strict, ignore_unexpected):
     if clazz is str:
         return __parse_type(value, clazz, path, isinstance(value, str))
 
+    if clazz is Any:
+        if type(value) is ConfigTree:
+            return dict(value)
+
+        return value
+
+    if isclass(clazz) and (issubclass(clazz, Enum) or issubclass(clazz, IntEnum)):
+        if isinstance(value, int):
+            return clazz.__call__(value)
+        elif isinstance(value, str):
+            return clazz.__getattr__(value)
+        else:
+            raise TypeConfigException(
+                f"expected str or int at {path}, got {type(value)}"
+            )
+
     if clazz is datetime:
         dt = __parse_type(value, clazz, path, isinstance(value, str))
         try:
@@ -179,21 +202,37 @@ def __parse(value: any, clazz, path, strict, ignore_unexpected):
     if clazz is relativedelta:
         return __parse_type(value, clazz, path, isinstance(value, relativedelta))
 
-    if clazz is ConfigTree:
-        return __parse_type(value, clazz, path, isinstance(value, ConfigTree))
-
     child_failures = []
+    child_successes = []
+    subtype = value.pop("_type", default=None)
     for child_clazz in sorted(clazz.__subclasses__(), key=lambda c: c.__name__):
-        if is_dataclass(child_clazz):
+        if is_dataclass(child_clazz) and (
+            subtype is None
+            or f"{child_clazz.__module__}.{child_clazz.__name__}".endswith(subtype)
+        ):
             try:
-                return __parse(value, child_clazz, path, strict, ignore_unexpected)
+                child_successes.append(
+                    (
+                        child_clazz,
+                        __parse(value, child_clazz, path, strict, ignore_unexpected),
+                    )
+                )
             except (
                 TypeConfigException,
                 MalformedConfigException,
                 UnexpectedKeysException,
+                AmbiguousSubclassException,
             ) as e:
                 child_failures.append(e)
 
+    if len(child_successes) == 1:
+        return child_successes[0][1]
+    elif len(child_successes) > 1:
+        matching_classes = "\n- ".join(map(lambda x: x[0].__name__, child_successes))
+        raise AmbiguousSubclassException(
+            f"""multiple subtypes of {clazz} matched at {path}, use '_type' to disambiguate:\n- {matching_classes}"""
+        )
+
     # no need to check length; false if empty
     if child_failures:
         failures = "\n- ".join([str(c) for c in child_failures])
@@ -204,7 +243,7 @@ def __parse(value: any, clazz, path, strict, ignore_unexpected):
     raise TypeConfigException(f"expected type {clazz} at {path}, got {type(value)}")
 
 
-def __generate(value: object, path):
+def __generate(value: object, path: str):
 
     if is_dataclass(value):
         tree = {k: __generate(v, f"{path}.{k}") for k, v in asdict(value).items()}
@@ -225,7 +264,7 @@ def __generate(value: object, path):
     return value
 
 
-def __dict_list_parsing(prefix: str, obj):
+def __env_vars_parse(prefix: str, obj: Dict[str, Any]):
     ret = {}
 
     def set_lens(p, focus, v):
@@ -299,3 +338,24 @@ def __dict_list_parsing(prefix: str, obj):
             set_lens(path, ret, v)
 
     return ret
+
+
+def __cli_parse(argv: List[str]):
+    kvs = {}
+    i = 0
+
+    if not argv[0].startswith("--"):
+        i += 1
+
+    while i < len(argv):
+        curr = argv[i]
+        if not curr.startswith("--"):
+            raise ParseException(f"cli argument {curr} is misplaced")
+
+        if i + 1 == len(argv):
+            raise ParseException("last argument is missing")
+
+        kvs[curr[2:].replace("-", "_")] = argv[i + 1]
+        i += 2
+
+    return __env_vars_parse("", kvs)
